package com.yunzhi.niucai.controller;

import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.payment.app.models.AlipayTradeAppPayResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.config.InterceptorConfig;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.service.ITaOrderService;
import com.yunzhi.niucai.service.ITaShopAppService;
import com.yunzhi.niucai.vo.LoginParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;


@Api(tags = "登入/登出")
@RestController
@RequestMapping("/aliPay")
public class AliPayController extends BaseController {

    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private ITaOrderService taOrderService;

    @Value("${aliPay.notifyUrl}")
    private String aliPayNotifyUrl;

    @PostMapping("/create")
    @ApiOperation(value = "创建订单", notes = "创建订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean create(HttpServletRequest request, @RequestBody TaOrder taOrder) throws Exception {
        Factory.setOptions(getOptions(taOrder));
        String outTradeNo = "ALI" + System.currentTimeMillis() + randomString(5);
        AlipayTradeAppPayResponse pay = Factory.Payment.App().pay("支付宝支付", outTradeNo, String.valueOf(taOrder.getAmount() / 100));
        // 更新商户订单号并改为支付中
        taOrder.setOutTradeNo(outTradeNo);
        taOrder.setStatus(CommConstants.SRTATUS_PAYING);
        taOrder.setUpdateDate(LocalDateTime.now());
        taOrderService.updateById(taOrder);
        return ResponseBean.success(pay);
    }

    public String randomString(int length) {
        String str = new BigInteger(130, new SecureRandom()).toString(32);
        return str.substring(0, length);
    }

    public Config getOptions(TaOrder taOrder) throws Exception {
        Config config = new Config();
        config.protocol = "https";
        config.gatewayHost = "openapi.alipay.com";
        config.signType = "RSA2";

        TaShopApp taShopApp = iTaShopAppService.getOne(new QueryWrapper<TaShopApp>().eq("shop_id", taOrder.getShopId()));
        config.appId = taShopApp.getAppid();
        // 私钥
        config.merchantPrivateKey = taShopApp.getMerchantPrivateKey();
        //支付宝公钥字符串
         config.alipayPublicKey = taShopApp.getAlipayPublicKey();
        //可设置异步通知接收服务地址
        config.notifyUrl = aliPayNotifyUrl;
        return config;
    }

    @GetMapping("/callback")
    public void callback(HttpServletRequest request){
        String tradeNo = request.getParameter("trade_no");
        String appId = request.getParameter("app_id");
        String outTradeNo = request.getParameter("out_trade_no");
        String tradeStatus = request.getParameter("trade_status");
        TaOrder taOrder = taOrderService.getOne(new QueryWrapper<TaOrder>().eq("out_trade_no", outTradeNo));
        taOrder.setUpdateDate(LocalDateTime.now());
        // 支付失败
        if (!tradeStatus.equals(CommConstants.TRADE_STATUS_SUCCESS)){
            taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
            taOrderService.updateById(taOrder);
            return;
        }
        //支付成功 更新为已支付状态
        taOrder.setIsPayed(true);
        taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
        taOrder.setTradeNo(tradeNo);
        taOrderService.updateById(taOrder);
    }
}
